import requests
# test_typer.py
from typer.testing import CliRunner
from llm_benchmark.main import app

#"model" : "gemma:2b"
#"model" : "gemma:7b"

def test_localhost_ollama():
    # This test is preventing WSL2 to connect to Windows version of Ollama
    url = "http://127.0.0.1:11434/"
    response = requests.get(url)
    rtext = response.text
    assert rtext.strip() == "Ollama is running"

def test_hello_world():
    runner = CliRunner()
    result = runner.invoke(app, ['hello','World'])
    assert result.exit_code == 0
    assert 'Hello World!' in result.output




import argparse
import yaml
import subprocess
import datetime
import pkg_resources

parser = argparse.ArgumentParser(
    prog="python3 check_models.py",
    description="Before running check_models.py, please make sure you installed ollama successfully \
        on macOS, Linux, or on Windows Powershell. You can check the website: https://ollama.com",
    epilog="Author: Jason Chuang")

parser.add_argument("-v",
                    "--verbose",
                    action="store_true",
                    help="this program helps you check whether you have ollama benchmark models installed")

parser.add_argument("-m",
                    "--models",
                    type=str,
                    help="provide benchmark models YAML file path. ex. ../data/benchmark_models.yml")

parser.add_argument("-b",
                    "--benchmark",
                    type=str,
                    help="provide benchmark YAML file path. ex. ../data/benchmark1.yml")

parser.add_argument("-t",
                    "--type",
                    type=str,
                    help="provide benchmark model type. ex, instruct")


def parse_yaml(yaml_file_path):
    with open(yaml_file_path, 'r') as stream:
        try:
            data=yaml.safe_load(stream)
            #print(d)
        except yaml.YAMLError as e:
            print(e)
    return data

def run_benchmark(models_file_path, benchmark_file_path, type, ollamabin: str = 'ollama'):
    
    models_dict = parse_yaml(models_file_path)
    benchmark_dict = parse_yaml(benchmark_file_path)
    model_type = type
    allowed_models = {e['model'] for e in models_dict['models']}
    
    ans={}
    if (model_type=='custom-model'):
        print("Running custom-model")
        #dynamically add models to benchmark_dict for custom-model
        for model in models_dict['models']:
            for one_model_type in benchmark_dict['modeltypes']:
                if one_model_type['type'] == 'custom-model':
                    if one_model_type['models'] is None:
                        one_model_type['models'] = []
                    one_model_type['models'].append(model)

    # Writing to file        
    for one_model_type in benchmark_dict['modeltypes']:
        #print(one_model_type)
        if (one_model_type['type']==model_type):
            #print(one_model_type['models'])
            #print(one_model_type['prompts'])
            for onemodel in one_model_type['models']:
                if (onemodel['model'] in allowed_models ):
                    loc_dt = datetime.datetime.today()
                    with open(f'log_{loc_dt.strftime("%Y-%m-%d-%H%M%S")}.log', "w", encoding='utf-8') as file1:
                        #print(onemodel)
                        stored_nums=[]
                        model_name = onemodel['model']
                        print(f'model_name =    {model_name}')
                        file1.write(f'\nmodel_name =    {model_name}\n')
                        
                        if model_name.startswith('llava'):
                            for one_prompt in one_model_type['prompts']:
                                img_file_names = one_prompt['keywords'].split(',')
                                for img in img_file_names:
                                    img_file_path = pkg_resources.resource_filename('llm_benchmark',f'data/img/{img}')
                                    prompt = f"{one_prompt['prompt']} {img_file_path}"
                                    print(f"prompt = {prompt}")
                                    result = subprocess.run([ollamabin, 'run', model_name, one_prompt['prompt'],'--verbose'], capture_output=True, text=True, check=True, encoding='utf-8')
                                    std_err = result.stderr
                                    #print(result.stderr)
                                    file1.write(std_err)
                                    
                                    for line in std_err.split('\n'):
                                        if ('eval rate' in line) and ('prompt' not in line):
                                            print(line)
                                            number = float(line[-20:-8])
                                            stored_nums.append(number)
                                            #print(number)
                        else:
                            for one_prompt in one_model_type['prompts']:
                                print(f"prompt = {one_prompt['prompt']}")
                                result = subprocess.run([ollamabin, 'run', model_name, one_prompt['prompt'],'--verbose'], capture_output=True, text=True, check=True, encoding='utf-8')
                                std_err = result.stderr
                                #print(result.stderr)                                   
                                file1.write(std_err)                                    
                                
                                for line in std_err.split('\n'):
                                    if ('eval rate' in line) and ('prompt' not in line):
                                        print(line)
                                        number = float(line[-20:-8])
                                        stored_nums.append(number)
                                        #print(number)

                        print("-"*20) 
                        if(len(stored_nums)!=0):       
                            average = sum(stored_nums)/len(stored_nums)
                            print("Average of eval rate: ", round(average,3), " tokens/s")
                            ans[f"{model_name}"]=f"{round(average,3):.2f}"

                        print("-"*40)
                        file1.write("\n"+"-"*40)
                    file1.close()
                    
    return ans

if __name__ == "__main__": 
    args = parser.parse_args()
    #print(f"args.verbose value：{args.verbose}")
    if (args.models is not None) and (args.benchmark is not None) and (args.type is not None):
        run_benchmark(args.models, args.benchmark, args.type, args.ollamabin)
        print('-'*40)
        
        


import typer, base64
from llm_benchmark import check_models
from llm_benchmark import check_ollama
from llm_benchmark import run_benchmark

from .systeminfo import sysmain

from .security_connection import connection 

import pkg_resources


app = typer.Typer()

@app.command()
def hello(name: str):
    print(f"Hello {name}!")
    

@app.command()
def run(ollamabin: str = 'ollama' , sendinfo : bool = True , custombenchmark : str = None):
    sys_info = sysmain.get_extra()
    print(f"Total memory size : {sys_info['memory']:.2f} GB") 
    print(f"cpu_info: {sys_info['cpu']}")
    print(f"gpu_info: {sys_info['gpu']}")
    print(f"os_version: {sys_info['os_version']}")

    ollama_version = check_ollama.check_ollama_version(ollamabin)
    print(f"ollama_version: {ollama_version}")
    print('-'*10)

    ft_mem_size = float(f"{sys_info['memory']:.2f}")

    if custombenchmark:
        models_file_path = custombenchmark
        sendinfo = False
        print(f"running custom benchmark from models_file_path: {models_file_path}")
        print(f"Disabling sendinfo for custom benchmark")
    else:
        models_file_path = pkg_resources.resource_filename('llm_benchmark','data/benchmark_models_32gb_ram.yml')
        if(ft_mem_size>=1 and ft_mem_size <2):
            models_file_path = pkg_resources.resource_filename('llm_benchmark','data/benchmark_models_2gb_ram.yml')
        elif(ft_mem_size>=2 and ft_mem_size <4):
            models_file_path = pkg_resources.resource_filename('llm_benchmark','data/benchmark_models_3gb_ram.yml')
        elif(ft_mem_size>=4 and ft_mem_size <7):
            models_file_path = pkg_resources.resource_filename('llm_benchmark','data/benchmark_models_4gb_ram.yml')
        elif(ft_mem_size>=7 and ft_mem_size <15):
            models_file_path = pkg_resources.resource_filename('llm_benchmark','data/benchmark_models_8gb_ram.yml')
        elif(ft_mem_size>=15 and ft_mem_size <31):
            models_file_path = pkg_resources.resource_filename('llm_benchmark','data/benchmark_models_16gb_ram.yml')

    check_models.pull_models(models_file_path)
    print('-'*10)

    benchmark_file_path = pkg_resources.resource_filename('llm_benchmark','data/benchmark2.yml')

    bench_results_info = {}
    is_simulation = False
    if custombenchmark:
        result0 = run_benchmark.run_benchmark(models_file_path,benchmark_file_path, 'custom-model', ollamabin)
        bench_results_info.update(result0)
    elif is_simulation==False :
        result1 = run_benchmark.run_benchmark(models_file_path,benchmark_file_path, 'instruct', ollamabin)
        bench_results_info.update(result1)
        result2 = run_benchmark.run_benchmark(models_file_path,benchmark_file_path, 'question-answer', ollamabin)
        bench_results_info.update(result2)
        result3 = run_benchmark.run_benchmark(models_file_path,benchmark_file_path, 'vision-image', ollamabin)
        bench_results_info.update(result3)
        result4 = run_benchmark.run_benchmark(models_file_path,benchmark_file_path, 'instruction-question-answer-code-generation', ollamabin)
        bench_results_info.update(result4)
    else:
        bench_results_info.update({"llama2:7b":7.65})
        bench_results_info.update({"gemma2:7b":17.77})

    if (sendinfo==True):
        print(f"Sending the following data to a remote server")
        print(f"Your machine UUID : {sysmain.get_uuid()}")
        #print(f"{bench_results_info.items()}")
        x = connection.send_benchmark(sysmain.get_uuid(),ollama_version,bench_results_info)
        #print(x)
        print('=='*10)
        #print(f"{sys_info.items()}")
        sys_info = sysmain.get_extra()
        sys_info['uuid']=f"{sysmain.get_uuid()}"
        x = connection.send_sysinfo(sys_info)
        #print(x)


@app.command()
def goodbye(name: str, formal: bool = False):
    if formal:
        print(f"Goodbye Mr.(Ms.) {name}. Have a good day.")
    else:
        print(f"Bye {name}!")

@app.command()
def sysinfo(formal: bool = True):
    if formal:
        sys_info = sysmain.get_extra()
        sys_info['uuid'] = f"{sysmain.get_uuid()}"
        #print(sys_info.items())
        print(f"memory : {sys_info['memory']:.2f} GB") 
        print(f"cpu_info: {sys_info['cpu']}")
        print(f"gpu_info: {sys_info['gpu']}")
        print(f"os_version: {sys_info['os_version']}")
        print(f"Your machine UUID : {sys_info['uuid']}")

        x = connection.send_sysinfo(sys_info)
        print(x)
    else:
        print(f"No print!")


if __name__ == "__main__":
    app()



from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='llm_benchmark',
    version='0.4.4',
    author='Jason Chuang',
    author_email='chuangtcee@gmail.com',
    description='LLM Benchmark',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/aidatatools/ollama-benchmark',
    packages=find_packages(),
    package_data={'llm_benchmark': ['data/*.yml', 'data/img/*.jpg']},
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.9',
    install_requires=[
        'click>=8.0.0',
        'typer==0.11.0',
        'ollama==0.2.1',
        'pyyaml==6.0.1',
        'requests==2.32.3',
        'psutil==5.9.8',
        'GPUtil==1.4.0',
        'lib-platform==1.2.10',
        'setuptools>=70.0.0'
    ],
    entry_points={
        'console_scripts': [
            'llm_benchmark = llm_benchmark.main:app',
        ],
    },
    # This line enables editable installs
    # With 'pip install -e .' equivalent
    # to install your package in editable mode
    # so changes in your source code are immediately reflected
    # without needing to reinstall
    options={'bdist_wheel': {'universal': False}},
    setup_requires=['setuptools>=70.0.0', 'wheel'],
    editable=True
)
